'httphelp.inc Copyright 2007 James J Keene
'Updated: July 19, 2007

DIM HTTPHELP_sock as SOCKET
$DEFINE peer HTTPHELP_sock

defstr http_dblcrlf = crlf+crlf

OBJECT HTTPHELP
  Host as STRING*32    'e.g., www.google.com
  Command as STRING*8 = "GET"
  Socket as LONG       'handle saved here
  Result as LONG
  ServerIP as STRING*16
  Server as STRING*32  'obtained from http header
  HeaderLength as LONG
  ContentLength as LONG
  ContentType as STRING*32
  LastModified as STRING*32
  Day as LONG
  Month as LONG
  Year as LONG
  Error as STRING*36
  Post as LONG      'optional objptr to POST Command data
  Progress as LONG  'optional codeptr to progress display
  Clear as SUB
  CloseSocket as SUB
  ConnectToServer as SUB
  Download as FUNCTION
  Header as FUNCTION
  Request as FUNCTION
END HTTPHELP

declare sub HTTPHELPClear
declare sub HTTPHELPCloseSocket
declare sub HTTPHELPConnectToServer
declare function HTTPHELPDownload _
  (http_resrc as STRING, lpMem as LONG) As LONG
declare function HTTPHELPHeader _
  (http_resrc as STRING, lpMem as LONG) As LONG
declare function HTTPHELPRequest _
  (http_resrc as STRING) as LONG

sub HTTPHELPClear
memset hbObj,0,sizeof(HTTPHELP)
end sub

sub HTTPHELPCloseSocket
if self.Socket then
  self.Result=peer.shutdown(self.Socket,2)
  self.Result=peer.close(self.Socket)
  self.Socket=zero
end if
end sub

sub HTTPHELPConnectToServer
defint r: defdbl time0
HTTPHELPCloseSocket
self.Error=null
if self.ServerIP=null then
  self.ServerIP=peer.AddrByName(self.Host)
end if
if self.ServerIP=null then
  self.Error="Host DNS lookup failed": Exit Sub
end if
defint s=peer.S
if s<=zero then self.Error="Could not create socket": Exit Sub
self.Socket=s
r=peer.NonBlock(s)
r=peer.Connect(s,self.ServerIP,80)
time0=TIMER
http_connect:
sleep 0.05
r=peer.IsConnected(s,0)
if r<=zero then
  if TIMER-time0<5.0 then goto http_connect 
  self.Error="Could not connect"
end if
self.Result=r
end sub

function HTTPHELPHeader
defint r: defdbl time0: defstr Buf
result=zero

HTTPHELPConnectToServer
if self.Error<>null then exit function

r=HTTPHELPRequest(http_resrc)
if not r then HTTPHELPCloseSocket: exit function
Buf.initialize 8192 'read buffer

use d as memory with lpMem
d.clear: time0=TIMER
http_recv:
Sleep 0.02
IF peer.IsServerReady(self.Socket)>zero THEN
  r=peer.Recv(self.Socket,Buf,8192)
  d.writestr Buf,r  'copyfrom(Buf,r)
  r=instr(d,http_dblcrlf)
  if not r then goto http_recv
  result=d.len: self.HeaderLength=r+3 
ELSE
  IF TIMER-time0<7.5 THEN goto http_recv
  result=zero
END IF

end_header:
self.Server=null
self.ContentLength=zero
self.ContentType=null
self.LastModified=null
self.Day=zero: self.Month=zero: self.Year=zero
if not d.len then goto end_use_d
if not instr(d,"200 OK") then goto end_use_d
'get header data into http object members
Buf=ucase$(d)
r=instr(Buf,"SERVER: ")
if r then d.position=r+7: self.Server=d.readline
r=instr(Buf,"CONTENT-LENGTH: ")
if r then d.position=r+15: self.ContentLength=val(d.readline)
r=instr(Buf,"CONTENT-TYPE: ")
if r then d.position=r+13: self.ContentType=d.readline
r=instr(Buf,"LAST-MODIFIED: ")
if r then d.position=r+14: self.LastModified=d.readline
Buf=ucase$(self.LastModified)
if Buf.len then
  self.Day=val(field$(Buf,space,2))
  self.Year=val(field$(Buf,space,4))
  Buf=field$(Buf,space,3)
  r=instr("JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC",Buf)
  if r then r=(r-one)\3+one
  self.Month=r
end if
d.position=d.len
end_use_d:
end use d
self.Result=zero
if self.Command="HEAD" then HTTPHELPCloseSocket
end function

function HTTPHELPDownload
defint k,r: defdbl time0: defstr Buf
result=zero

if self.Command<>"POST" then self.Command="GET"
r=HTTPHELPHeader(http_resrc,lpMem)
if not r then goto http_close
k=self.ContentLength
IF NOT k THEN goto http_close

use d as memory with lpMem
d=delete$(d,1,self.HeaderLength)  'delete http header
'get the rest of download
Buf.initialize 8192
http_download:
r=self.Progress
if r then funccall r,d.len,k
$IFNDEF CONSOLE
DOEVENTS
$ENDIF
IF d.len<k THEN
  time0=TIMER
  http_get:
  Sleep 0.02
  IF peer.IsServerReady(self.Socket)>zero THEN
    r=peer.Recv(self.Socket,Buf,8192)
    d.writestr(Buf,r)
    Goto http_download
  ELSE
    IF TIMER-time0<7.5 THEN Goto http_get
    self.Error="Download timed out before completion"
  END IF
ELSE
  result=d.len
END IF
end use d

http_close:
HTTPHELPCloseSocket
end function

function HTTPHELPRequest
defint p: defstr req
if self.Command="POST" and self.Post<>zero then _
 p=self.Post else p=zero
req.additems self.Command+space+http_resrc+" HTTP/1.1"
req.additems "Host: "+self.Host
req.additems "User-Agent: Mozilla/5.0"
if p then req.additems "Content-Length: "+str$(byref(p+8))
req.additems "Connection: close",null
if p then
  use d as memory with p
  req.write(d)
  end use d
end if
result=peer.Write(self.Socket,req,req.len)
end function

'This code can handle "small" POST data sizes
'probably shorter than the socket layer write buffer.
'POST with larger data size (d above) would need a 
'loop above to write all the data as it is sent.

$UNDEF peer
